Attribute VB_Name = "Gods_ac"
Option Explicit
'' """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
''    Copyright  1998 - 2003 Agilent Technologies Inc. All rights
''    reserved.
''
'' You have a royalty-free right to use, modify, reproduce and distribute
'' the Sample Application Files (and/or any modified version) in any way
'' you find useful, provided that you agree that Agilent has no
'' warranty,  obligations or liability for any Sample Application Files.
''
'' Agilent Technologies provides programming examples for illustration only,
'' This sample program assumes that you are familiar with the programming
'' language being demonstrated and the tools used to create and debug
'' procedures. Agilent support engineers can help explain the
'' functionality of Agilent software components and associated
'' commands, but they will not modify these samples to provide added
'' functionality or construct procedures to meet your specific needs.
'' """"""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""

'Constants
Global Const AppName = "Gods AC"
Global Const Lvfilter = 120000              'LOW VOLTAGE INPUT FILTER B.W.
Global Const Hvattn = 36000                  'HIGH VOLTAGE ATTENUATOR B.W.(NUMERATOR)
Global Const Gain100bw = 82000              'AMP GAIN 100 B.W. PEAKING CORRECTION!
Global Const PI = 3.14159265358979

' ----------Default values
Public Const def_ForceFreq = 0#      ' 1= input freq. of signal, 0 = auto measure
Public Const def_Force = 0           ' 1 = force sampling parameters, 0= auto
Public Const def_Tsamp = 0.001
Public Const def_Aper = def_Tsamp - 0.00003
Public Const def_Num = 800
Public Const def_Aper_Targ = 0.001   ' A/D aperture target (sec)
Public Const def_Nharm = 6     ' Minimum # harmonics sampled before alias
Public Const def_Nbursts = 6       'Number of bursts used for each measurement
Public Const def_Meas_Time = 15    ' the target measurement time in seconds

'-----------LESS COMMONLY MODIFIED VARIABLES
Global Forcefreq As Long    '1=INPUT FREQ. OF SIGNAL, 0= AUTOMATIC IF>1Hz
Global Force As Integer     '1=FORCE SAMP. PARAMETERS, 0= AUTOMATIC
Global Tsampforce As Double ' FORCED PARAMETER
Global Aperforce As Double  ' FORCED PARAMETER
Global Numforce As Long     ' FORCED PARAMETER
Global m_FreqWarning As String ' Message when frequ is not automatic
Global UserFrequency        ' Frequency user inputs
Global num As Long          ' number of samples
Global Aper_targ As Double  'A/D APERTURE TARGET (SEC)
Global NharmUser As Long    ' Min # harmonics before aliasing requested by user
Global Nharm As Long        'MINIMUM # HARMONICS SAMPLED BEFORE ALIAS
Global Nbursts As Long      'NUMBER OF BURSTS USED FOR EACH MEASUREMENT
Global freq As Double
Global TSamp As Double
Global bw_corr As Double
Global PPMerror As Double          ' Total error estimate
Global aper As Double
Global sinc As Double
Global dist_er As Double
Global ReadingCount As Integer ' counts the number of passes for a reading

'----------- MODIFIABLE VARIABLES
Public Meas_time As Long         'TARGET MEASUREMENT TIME (SEC)
Global Expect As Double    ' the expected ac results, ( an input guess)


Global range As Double      ' DMM range value

Dim m_cancel As Boolean

'GODS_AC"
'PROGRAM MEASURES LOW FREQ RMS VOLTAGES (<1KHz)  2/27/98, updated 12/19/2002
'CAN BE ACCURATE TO 0.001% IF Meas_time>30 Sec
'Translated from an HP Basic program by Ron Swerlein
'Author; Werner Haussmann in VB5.0, VB6.0
'
'NOTES:
'1.  DISTORTED SINEWAVES HAVE HIGHER FREQ HARMONICS
'    THAT MAY NOT BE MEASURED IF MEASUREMENT BANDWIDTH IS TOO LOW.
'    COMPUTED ERROR IS INCLUDED FOR UP TO 1% HARMONIC DISTORTION.
'    THIS ERROR CAN BE REDUCED BY USING SMALLER Aper_targ(LINE 360)
'2.  ESPECIALLY AT LOW SIGNAL LEVELS, TWO AC VOLTMETERS WITH "PERFECT"
'    ACCURACY MAY READ DIFFERENTLY IF THEY HAVE DIFFERENT
'    NOISE BANDWIDTHS.  THIS IS TRUE ONLY IF THE SIGNAL BEING MEASURED
'    CONTAINS APPRECIABLE HIGH FREQUENCY NOISE OR SPURIOUS SIGNALS.
'    THIS PROGRAM DISPLAYS THE MEASUREMENT BANDWIDTH WHEN RUN.
'    THIS BANDWIDTH VARIES DEPENDING ON Freq.  IT IS APPROX= .5/Aper_targ.
'    ALSO, MAKING Nharm(LINE 370) HIGHER CAN INCREASE BANDWIDTH, BUT
'    CAN HURT BASIC ACCURACY BY FORCING SMALL A/D APERTURES.
'
Public Sub startprogram(dmm As AgtIOServer)
   Dim continue As Boolean
   
   On Error Resume Next
   If isconnected(dmm) = False Then Exit Sub
   
   ' increase timeout for IO
   dmm.Timeout = (Meas_time / Nbursts) * 1000 + 10000
   
   frmMain.rtbHistory.Text = ""
   continue = InitParameters(dmm)
   If Not continue Then
      Exit Sub
   End If
   frmMain.Refresh
   frmMain.lblACDC.Caption = ""
   frmMain.lblACrms.Caption = ""
   frmMain.lblACDC.Refresh
   frmMain.lblACrms.Refresh
   continue = setupHP3458A(dmm)
   If Not continue Then
      Exit Sub
   End If
   PreComputation
   WriteToRichTextBox
   WriteTodialog
   DoEvents
   frmMain.rtbHistory.SetFocus
   frmMain.rtbHistory.SelText = vbCrLf & "Begin first Reading, " _
                           & "time; " & Format(Now, "short Time") _
                           & vbCrLf
   ReadingCount = 1
   startGodsAC dmm
   
End Sub
Public Sub repeatReading(dmm As AgtIOServer)
   If isconnected(dmm) Then
   ReadingCount = ReadingCount + 1
   frmMain.lblACDC.Caption = ""
   frmMain.lblACrms.Caption = ""
   frmMain.lblACDC.Refresh
   frmMain.lblACrms.Refresh
   frmMain.rtbHistory.SelText = vbCrLf & "Begin Reading " & ReadingCount _
                           & ", time = " & Format(Now, "short Time") _
                           & vbCrLf
   startGodsAC dmm
    
    End If
End Sub

Function InitParameters(dmm As AgtIOServer) As Boolean
' Initialize the parameters that define the measurement
' returns false if user cancels or there is a problem
   Dim msg As String
   Dim cancelmsg As String
   Dim replyString As String
   Dim number As Double
   
   On Error Resume Next
   
   msg = "Enter expected rms value of" & vbCrLf _
         & "signal to be measured within 10% :" & vbCrLf _
         & "Smallest measureable voltage is 1mV, largest is 700V"
   'INPUT "ENTER EXPECTED RMS VALUE OF SIGNAL TO BE MEASURED +-10%",Expect
   msg = InputBox(msg, AppName)
   ' Check for valid value
   If IsNumeric(msg) Then
      Expect = CDbl(msg)
      If Expect < 0.001 Then msg = ""
      If Expect > 700 Then msg = ""
   Else
      msg = ""
   End If
   cancelmsg = "No entry, out of bounds or user canceled." & vbCrLf _
            & "Start over to make a measurement"
   If msg = "" Then
      MsgBox cancelmsg, vbOKOnly + vbCritical, AppName
      InitParameters = False
      Exit Function
   End If
   Expect = Val(msg)
   Expect = 1.55 * Expect    'PEAK VALUE *1.1
   If Expect < 0.12 Then
     range = 0.1
   End If
   If Expect >= 0.12 And Expect < 1.2 Then
     range = 1
   End If
   If Expect >= 1.2 And Expect < 12 Then
      range = 10
   End If
   If Expect >= 12 And Expect < 120 Then
      range = 100
   End If
   If Expect >= 120 Then
     range = 1000
   End If
   With dmm
      .Output "RESET"
      .Output "END ALWAYS"
      .Output "DCV 1000"
      
      Beep
      .Output "DISP OFF,***GODS_AC*** "
      msg = "Apply input signal and let settle"
      MsgBox msg, vbInformation, AppName
      .Output "DISP OFF,***GODS_AC***"
   End With 'dmm
   
   If Forcefreq = 1 Then         'IF MANUALLY ENTERING FREQUENCY
     Beep
     replyString = InputBox("Enter the Frequency", AppName, Str$(UserFrequency))
     freq = Val(replyString)
     If freq > 0 Then m_FreqWarning = "WARNING! Frequency forced"
   Else
      freq = Frequency(Expect, dmm) 'GET INPUT SIGNAL FREQUENCY
   End If 'forcefreq = 1
   
   If freq = 0 Then
      msg = "Frequency value is 0" & vbCrLf _
            & "Restart the measurement."
      MsgBox msg, vbOKOnly + vbCritical, AppName
      InitParameters = False
      Exit Function
   End If 'freq
   
   If Force = 1 Then             'IF NOT GETTING PARAMETERS AUTOMATICALLY
      TSamp = Tsampforce
      aper = Aperforce
      num = Numforce
   Else                          'AUTOMATICALLY GET SAMPLING PARAMETERS
      InitParameters = Samp_parm(TSamp, aper, num, freq, Meas_time, _
                                    Aper_targ, NharmUser, Nbursts)
      Exit Function
   End If
   InitParameters = True
   
End Function ' init parameters

Function setupHP3458A(dmm As AgtIOServer) As Boolean
'------- SETUP HP3458A
    Dim reply As String
    Dim strTemp As String
    Dim msg As String
    Dim Storage As Long

    strTemp = Str$(range)
    With dmm
        .Output "TARM HOLD;AZERO OFF;DCV " & Str$(range)
        .Output "APER " & Str$(aper) & _
                ";NRDGS " & Str$(num) & _
                ",TIMER"
        .Output "TIMER " & Str$(TSamp)
        .Output "TRIG LEVEL;LEVEL 0,DC;DELAY 0;LFILTER ON"
        .Output "MSIZE?"
        .Enter reply
    End With
    Storage = Val(reply)
    Storage = Int(Storage / 4)                             'STORAGE CAPACITY IN VOLTMETER (DINT DATA)
    setupHP3458A = True
    If num > Storage Then
        Beep
        msg = "Not enough voltmeter memory" & vbCrLf _
                & "for needed samples" & vbCrLf _
                & "Try a larger Aperture Target," & vbCrLf _
                & "or smaller Number of samples."
        MsgBox msg, vbOKOnly + vbCritical, AppName
        setupHP3458A = False
    End If
    Debug.Print "end of setup HP3458a"
End Function
Sub PreComputation()
   Dim X As Double
   Dim msg As String
      
 '------ RELIMINARY COMPUTATIONS
   X = PI * aper * freq
   sinc = Sin(X) / X              'USED TO CORRECT FOR A/D APERTURE ERROR
   bw_corr = VoltMeter_bw(freq, range) 'USED TO CORRECT FOR Vmeter BANDWIDTH
   ErrorEstimate PPMerror, dist_er, freq, range, num, aper, Nbursts 'MEASUREMENT UNCERTAINTY
   If Force = 1 Then
     msg = "****** PARAMETERS ARE FORCED, ACCURACY MAY BE DEGRADED! ******"
     MsgBox msg, vbOKOnly
     frmMain.rtbHistory.SelText = "NOTE: Prameters are forced, Accuracy may be degraded." _
                                 & vbCrLf
     
   End If
End Sub
Sub WriteToRichTextBox()
   With frmMain.rtbHistory
      .SelText = vbCrLf & Format(Now, "long Date") _
                           & vbCrLf & vbCrLf
      .SelText = "Signal Frequency = " & vbTab & Format$(freq, "#\ ###.000\ 0") & " Hz"
      .SelText = vbCrLf
      .SelText = m_FreqWarning
      m_FreqWarning = ""
      .SelText = vbCrLf
      .SelText = "Number of bursts = " & Str$(Nbursts) & " bursts"
      .SelText = vbCrLf
      .SelText = "Number of samples = " & Str$(num) & " samples/burst"
      .SelText = vbCrLf
      .SelText = "Sample spacing = " & Format$(TSamp * 1000, "####0.000\ 0") & " msec"
      .SelText = vbCrLf
      .SelText = "A/D Aperture = " & Format$(aper * 1000, "###0.000\ 0") & " msec"
      .SelText = vbCrLf
      .SelText = "Measurement bandwidth = " & Str$(Int(5 / aper) / 10) & " Hz"
      .SelText = vbCrLf
      .SelText = "Estimated total sinewave measurement uncertainty = " & Str$(PPMerror) & " PPM"
      .SelText = vbCrLf
      .SelText = "Additional error for 1% distortion " & vbCrLf _
                  & "(3rd Harmonic distortion) = " & Str$(dist_er) & " PPM"
      .SelText = vbCrLf
      .SelText = "NOTE: Error estimate assumes calibration" & vbCrLf _
                  & "(ACAL DCV) Performed recently (<24hrs)" & vbCrLf
      .Refresh
   End With 'frmMain.rtbHistory
End Sub
Sub WriteTodialog()
   Dim temp As Double
   
   With frmMain
      .lblSampleSpace.Caption = Format$(TSamp * 1000, "####0.000\ 0") ' display in msec
      .lblNBursts.Caption = Format$(Nbursts)
      .lblNum.Caption = Format$(num)
      .lblSigFreq.Caption = Format$(freq, "#,###.000\ 0")
      .lblAperture.Caption = Format$(aper * 1000, "####0.000\ 0") ' display in msec
      temp = Int(5 / aper) / 10
      .lblBW.Caption = Format$(temp)
      .lblSineUncertainty.Caption = Format$(PPMerror, "####")
      .lblHarmonicUncertain.Caption = Format$(dist_er, "####")
  
   End With 'frmMain.rtbHistory
   
End Sub
Sub startGodsAC(dmm As AgtIOServer)
    Dim sum, sumsq As Double
    Dim temp As Double
    Dim delay As Double
    Dim mean As Double
    Dim sdev As Double
    Dim timeBegin, timeEnd As Date
    Dim deltaSeconds As Double
    Dim i As Integer
    Dim DCrms, ACrms, DC As Double
    Dim strTemp As String

    On Error GoTo GODSACerror
    
    m_cancel = False
    
    dmm.Output "DISP OFF,***GODS_AC***"
    timeBegin = Now
    sum = 0
    sumsq = 0
    With frmMain
        .rtbHistory.SelText = vbTab & "The " & Nbursts _
                & " intermediate results:" & vbCrLf
        .txtIntermediate.Text = ""
        .txtIntermediate.Refresh
        
        For i = 0 To Nbursts - 1
        
            DoEvents
            If m_cancel Then GoTo UserCanceled
            
            delay = i / Nbursts / freq + 0.000001
            dmm.Output "DELAY " & Str$(delay)
            dmm.Output "TIMER " & Str$(TSamp)
            MeasureAC mean, sdev, dmm                      ' make the measurement and return stats
            
            If m_cancel Then GoTo UserCanceled
            
            sumsq = sumsq + sdev * sdev + mean * mean
            sum = sum + mean
            temp = sdev * bw_corr / sinc                   'CORRECT A/D Aper AND dmm B.W.

            .txtIntermediate.SelText = Str$(i + 1) & "     " & FormatReading$(temp) & vbCrLf
            .rtbHistory.SelText = vbTab & Str$(i + 1) & vbTab _
                    & FormatReading(temp) & vbCrLf
        Next i

        DoEvents
        If m_cancel Then GoTo UserCanceled

        DCrms = Sqr(sumsq / Nbursts)
        DC = sum / Nbursts
        ACrms = DCrms * DCrms - DC * DC
        If ACrms < 0 Then ACrms = 0                        'PROTECTION FOR SQR OF NEG NUMBER
        ACrms = Sqr(ACrms)
        ACrms = ACrms * bw_corr / sinc                     'CORRECT A/D Aper AND dmm B.W.
        DCrms = Sqr(ACrms * ACrms + DC * DC)
        timeEnd = Now
        deltaSeconds = (timeEnd - timeBegin) * 86400
        strTemp = "Time for complete measurement, (hrs:min:sec): " _
                & Format$(timeEnd - timeBegin, "h:nn:ss")

        .lblTime.Caption = Format$(deltaSeconds, "#######.0")
        .lblACrms.Caption = FormatReading(ACrms)
        .lblACDC.Caption = FormatReading(DCrms)
        .rtbHistory.SelText = vbCrLf & "Measured Voltages: " _
                & vbCrLf _
                & vbTab & "AC rms" & vbTab & vbTab & "= " & FormatReading(ACrms) & vbCrLf _
                & vbTab & "AC+DC rms " & vbTab & "= " & FormatReading(DCrms) & vbCrLf
        .rtbHistory.SelText = vbTab & strTemp & vbCrLf
        .rtbHistory.Refresh
    End With                                               'frmMain

    '********************** PRINT RMS VALUES ***************
    strTemp = "DISP OFF, " & "'" & Format(ACrms, "####.000\,000\,00") & " VAC'"
    dmm.Output strTemp

    Exit Sub
    
GODSACerror:
    frmMain.rtbHistory.SelText = vbCrLf & "WARNING: Error in measurement" & vbCrLf
    m_cancel = True
    dmm.Clear
    dmm.Output "DISP OFF, **ERROR**"
    Exit Sub
UserCanceled:
    MsgBox "Measurement Canceled", vbCritical
    frmMain.rtbHistory.SelText = vbCrLf & "WARNING: Measurement Canceled" & vbCrLf
    dmm.Clear
    
    dmm.Output "DISP OFF, *MEAS_CANCELED*"
End Sub
Function FormatReading(ByVal n As Double) As String
   If n > 110# Then
      FormatReading = Format$(n, "###.000\ 0")
   ElseIf n > 1.1 Then
      FormatReading = Format$(n, "###.000\ 000")
   ElseIf n > 0.1 Then
      FormatReading = Format$(n, "#.000\ 000\ 0")
   ElseIf n > 0.01 Then
      FormatReading = Format$(n, "#.000\ 000\ 00")
   Else
      FormatReading = Format$(n, "#.000\ 000\ 000")
   End If
   
End Function

 Sub MeasureAC(mean As Double, sdev As Double, dmm As AgtIOServer)
    Dim temp As String
    Dim ds As Double

    On Error GoTo ErrorMeasAC
    
    If m_cancel Then GoTo UserCanceled

    ds = Meas_time / Nbursts
    
    '###
    'vmeter.CommTimeout = ds * 1000 + 10000

    With dmm
        .Output "MEM FIFO;MFORMAT DINT;TARM SGL"
        delay 50
        .Output "MMATH STAT"
        delay 50
        .Output "RMATH SDEV"
        delay 50
        .Enter sdev
        .Output "RMATH MEAN"
        .Enter mean
        sdev = Sqr(sdev * sdev * (num - 1) / num)          ' SDEV FORMULA
    End With
    
    Exit Sub
UserCanceled:
    Exit Sub
ErrorMeasAC:
        MsgBox "Error in Measure AC"
    m_cancel = True
    End Sub

Function Frequency(Expect As Double, dmm As AgtIOServer) As Double
    Dim strExpect As String
    Dim Cal As Double
    Dim freq As Double
    Dim msg, reply As String

    On Error Resume Next

    strExpect = Str$(Expect * 0.9)
    With dmm
        .Output "TARM HOLD;LFILTER ON;LEVEL 0,DC;FSOURCE ACDCV"
        .Output "FREQ " & strExpect
        .Output "CAL? 245"
        .Enter Cal
        .Output "TARM SGL"
        delay 200
        .Enter freq
    End With
    
    'UNCALIBRATED FREQUENCY IS USED FOR MORE ACCURATE SAMPLE DETERMINATION
    'SINCE TIMER IS UNCALIBRATED
    freq = freq / Cal
    If freq = 0 Then
        Beep
        reply = InputBox("Frequency Measurement was 0," & vbCrLf & "Please Enter the frequency", AppName)
        If reply = "" Then
            Frequency = 0
            Exit Function
        End If
        freq = Val(reply)
        If freq > 1 Then
            Beep
            msg = "WARNING!! AUTOMATIC FREQUENCY MEASUREMENT SHOULD HAVE WORKED"
            MsgBox msg, vbOKOnly, AppName
            m_FreqWarning = "WARNING! Automatic frequency did not work"
        End If
    End If
    Frequency = freq
    Debug.Print "end of frequency measure"
End Function

Function VoltMeter_bw(ByVal freq As Double, ByVal range As Double)
   Dim bw_corr As Double

   If range <= 0.12 Then
      bw_corr = (1 + (freq / Lvfilter) ^ 2) / (1 + (freq / Gain100bw) ^ 2)
      bw_corr = Sqr(bw_corr)
   End If
   If range > 0.12 And range <= 12 Then
      bw_corr = (1 + (freq / Lvfilter) ^ 2)
      bw_corr = Sqr(bw_corr)
   End If
   If range > 12 Then
      bw_corr = (1 + (freq / Hvattn) ^ 2)
      bw_corr = Sqr(bw_corr)
   End If
   VoltMeter_bw = bw_corr
End Function

Sub ErrorEstimate(PPMerror, Dist, freq, range, num, aper, Nbursts)
   Dim base As Integer
   Dim Vmeter_bw As Integer
   Dim aper_er, sincerr, harm_er, _
      Tim_er, Limit As Integer
   Dim df_err As Double   'Df_err IS THE ERROR DUE TO THE DISSIPATION FACTOR OF THE P.C. BOARD
   Dim Noise As Double
   Dim sinc, sinc2 As Double
   Dim Rsource, Cload, Df As Double
   Dim X1 As Double
   Dim X2 As Double
   Dim X, Y As Double
   Dim Noiseraw As Double
   
   
  'Base IS THE BASIC NPLC 100 DCV 1YR ACCURACY, THIS NUMBER CAN BE
  'SUBSTANTIALLY LOWER FOR HIGH STABILITY OPTION AND 90DAY CAL CYCLES
     If range > 120 Then             'SELF HEATING +BASE ERROR
       base = 15
     Else
       base = 10                     'BASIC 1YR ERROR(ppm)
     End If
  '
  '
  'Vmeter_bw IS ERROR DUE TO UNCERTAINTY IN KNOWING THE HIGH FREQUENCY
  'RESPONSE OF THE DCV FUNCTION FOR VARIOUS RANGES AND FREQUENCIES
  'UNCERTAINTY IS 30% AND THIS ERROR IS RANDOM
   X1 = VoltMeter_bw(freq, range)
   X2 = VoltMeter_bw(freq * 1.3, range)
   Vmeter_bw = Int(1000000# * Abs(X2 - X1)) 'ERROR DUE TO METER B.W.
  '
  '
  'Aper_er IS THE DCV GAIN ERROR FOR VARIOUS A/D APERTURES
  'THIS ERROR IS SPECIFIED IN A GRAPH ON PAGE 11 OF THE DATA SHEET
  'THIS ERROR IS RANDOM
     aper_er = Int(1# * 0.002 / aper) 'GAIN UNCERTAINTY - SMALL A/D APERTURE
     If aper_er > 30 And aper >= 0.00001 Then
       aper_er = 30
     End If
     If aper < 0.00001 Then
       aper_er = 10 + Int(0.0002 / aper)
     End If
  '
  '
  'Sincerr IS THE ERROR DUE TO THE APERTURE TIME NOT BEING PERFECTLY KNOWN
  'THIS VARIATION MEANS THAT THE Sinc CORRECTION TO THE SIGNAL FREQUENCY
  'IS NOT PERFECT.  ERROR COMPONENTS ARE CLOCK FREQ UNCERTAINTY(0.01%)
  'AND SWITCHING TIMING (50ns).  THIS ERROR IS RANDOM.
     X = PI * aper * freq
     sinc = Sin(X) / X
     Y = PI * freq * (aper * 1.0001 + 0.00000005)
     sinc2 = Sin(Y) / Y
     sincerr = Int(1000000# * Abs(sinc2 - sinc)) 'APERTURE UNCERTAINTY ERROR
  '
  '
  'Dist IS ERROR DUE TO UP TO 1% DISTORTION OF THE INPUT WAVEFORM
  'IF THE INPUT WAVEFORM HAS 1% DISTORTION, THE ASSUMPTION IS MADE
  'THAT THIS ENERGY IS IN THE THIRD HARMONIC.  THE APERTURE CORRECTION,
  'WHICH IS MADE ONLY FOR THE FUNDAMENTAL FREQUENCY WILL THEN BE
  'INCORRECT.  THIS ERROR IS RETURNED SEPERATELY.
    X = PI * aper * freq
     sinc = Sin(X) / X
     sinc2 = Sin(3 * X) / 3 / X 'SINC CORRECTION NEEDED FOR 3rd HARMONIC
     harm_er = Abs(sinc2 - sinc)
     Dist = Sqr(1 + (0.01 * (1 + harm_er)) ^ 2) - Sqr(1 + 0.01 ^ 2)
     Dist = Int(Dist * 1000000#)
  '
  '
  'Tim_er IS ERROR DUE TO MISTIMING.  IT CAN BE SHOWN THAT IF A
  'BURST OF Num SAMPLES ARE USED TO COMPUTE THE RMS VALUE OF A SINEWAVE
  'AND THE SIZE OF THIS BURST IS WITHIN 50ns*Num OF AN INTEGRAL NUMBER
  'OF PERIODS OF THE SIGNAL BEING MEASURED, AN ERROR IS CREATED
  'BOUNDED BY 100ns/4/Tsamp.  THIS ERROR IS DUE TO THE 100ns QUANTIZATION
  'LIMITATION OF THE HP3458A TIME BASE.  IF THIS ERROR WERE ZERO, THEN
  'Num*Tsamp= INTEGER/Freq, BUT WITH THIS ERROR UP TO 50ns OF TIMEBASE
  'ERROR IS PRESENT PER SAMPLE, THEREFORE TOTAL TIME ERROR=50ns*Num
  'THIS ERROR CAN ONLY ACCUMULATE UP TO 1/2 *Tsamp, AT WHICH POINT THE
  'ERROR IS BOUNDED BY 1/4/Num
  'THIS ERROR IS FURTHER REDUCED BY USING THE LEVEL TRIGGER
  'TO SPACE Nbursts AT TIME INCREMENTS OF 1/Nbursts/Freq.  THIS
  'REDUCTION IS SHOWN AS 20*Nbursts BUT IN FACT IS USUALLY MUCH BETTER
  'THIS ERROR IS ADDED ABSOLUTELY TO THE PPMerror CALCULATION
     Tim_er = Int(1000000# * 0.0000001 / 4 / (aper + 0.00003) / 20) 'ERROR DUE TO HALF CYCLE ERROR
     Limit = Int(1000000# / 4 / num / 20)
     If Tim_er > Limit Then Tim_er = Limit
  '
  '
  'Noise IS THE MEASUREMENT TO MEASUREMENT VARIATIONS DUE TO THE
  'INDIVIDUAL SAMPLES HAVING NOISE.  THIS NOISE IS UNCORRELATED AND
  'IS THEREFORE REDUCED BY THE SQUARE ROOT OF THE NUMBER OF SAMPLES
  'THERE ARE Nbursts*Num SAMPLES IN A MEASUREMENT.  THE SAMPLE NOISE IS
  'SPECIFIED IN THE GRAPH ON PAGE 11 OF THE DATA SHEET.  THIS GRAPH
  'SHOWS 1 SIGMA VALUES, 2 SIGMA VALUES ARE COMPUTED BELOW.
  'THE ERROR ON PAGE 11 IS EXPRESSED AS A % OF RANGE AND IS MULTIPLIED
  'BY 10 SO THAT IT CAN BE USED AS % RDG AT 1/10 SCALE.
  'ERROR IS ADDED IN AN ABSOLUTE FASHION TO THE PPMerror CALCULATION SINCE
  'IT WILL APPEAR EVENTUALLY IF A MEASUREMENT IS REPEATED OVER AND OVER
     Noiseraw = 0.9 * Sqr(0.001 / aper) '1 SIGMA NOISE AS PPM OF RANGE
     Noise = Noiseraw / Sqr(Nbursts * num) 'REDUCTION DUE TO MANY SAMPLES
     Noise = 10 * Noise            'NOISE AT 1/10 FULL SCALE
     Noise = 2 * Noise             '2 SIGMA
     If range <= 0.12 Then         'NOISE IS GREATER ON 0.1 V RANGE
       Noise = 7 * Noise           'DATA SHEET SAYS USE 20, BUT FOR SMALL
       Noiseraw = 7 * Noiseraw     'APERTURES, 7 IS A BETTER NUMBER
    End If
     Noise = Int(Noise) + 2            'ERROR DUE TO SAMPLE NOISE
  '
  '
  'Df_err IS THE ERROR DUE TO THE DISSIPATION FACTOR OF THE P.C. BOARD
  'CAPACITANCE LOADING DOWN THE INPUT RESISTANCE.  THE INPUT RESISTANCE
  'IS 10K OHM FOR THE LOW VOLTAGE RANGES AND 100K OHM FOR THE HIGH VOLTAGE
  'RANGES (THE 10M OHM INPUT ATTENUATOR).  THIS CAPACITANCE HAS A VALUE
  'OF ABOUT 15pF AND A D.F. OF ABOUT 1.0%.  IT IS SWAMPED BY 120pF
  'OF LOW D.F. CAPACITANCE (POLYPROPALENE CAPACITORS) ON THE
  'LOW VOLTAGE RANGES WHICH MAKES FOR AN EFFECTIVE D.F. OF ABOUT .11%.
  'THIS CAPACITANCE IS SWAMPED BY 30pF LOW D.F. CAPACITANCE ON THE
  'HIGH VOLTAGE RANGES WHICH MAKES FOR AN EFFECTIVE D.F. OF .33%.
  'THIS ERROR IS ALWAYS IN THE NEGATIVE DIRECTION, SO IS ADDED ABSOLUTELY
     If range <= 12 Then
       Rsource = 10000
       Cload = 0.000000000133
       Df = 0.0011     '0.11%
     Else
       Rsource = 100000
       Cload = 0.00000000005
      Df = 0.0033     '0.33%
     End If
     df_err = 2 * PI * Rsource * Cload * Df * freq
     df_err = Int(1000000# * df_err) 'ERROR DUE TO TO PC BOARD DIELECTRIC ABSORBTION
  '
  '
  'PPMerror IS TOTAL ERROR ESTIMATION.  RANDOM ERRORS ARE ADDED IN RSS FASHION
     PPMerror = Sqr(base ^ 2 + Vmeter_bw ^ 2 + aper_er ^ 2 + sincerr ^ 2)
     PPMerror = Int(PPMerror + df_err + Tim_er + Noise) 'TOTAL ERROR (ppm)
  '
End Sub


Function Samp_parm(TSamp, aper, num, freq, Meas_time, ByVal Aper_targ, _
                    ByVal Nharm_min, ByVal Nbursts) As Boolean
                    
   Dim submeas_time, burst_time As Double
   Dim TSampTemp As Double
   Dim approxnum, Ncycle As Long
   Dim K As Long
   Dim msg As String
   
   Samp_parm = True ' true for parameters are OK
   aper = Aper_targ
   TSamp = 0.0000001 * Int((aper + 0.00003) / 0.0000001 + 0.5) 'ROUND TO 100ns
   submeas_time = Meas_time / Nbursts         'TARGET TIME PER BURST
   'IT TAKES 1.5ms FOR EACH SAMPLE TO COMPUTE Sdev
   burst_time = submeas_time * TSamp / (0.0015 + TSamp)
   approxnum = Int((burst_time / TSamp) + 0.5)
   Ncycle = Int(burst_time * freq + 0.5)      '# OF 1/Freq TO SAMPLE
   If Ncycle = 0 Then
      Ncycle = 1
      TSamp = 0.0000001 * Int(1 / freq / approxnum / 0.0000001 + 0.5) 'TIME BETWEEN SAMPLES
      Nharm = Int(1 / TSamp / 2 / freq) '# HARMONICS BEFORE ALIAS OCCURS
      If Nharm < Nharm_min Then  'NEED TO INCREASE SAMPLE FREQUENCY
         Nharm = Nharm_min
         TSamp = 0.0000001 * Int(1 / 2 / Nharm / freq / 0.0000001 + 0.5)
      End If
   Else
      Nharm = Int(1 / TSamp / 2 / freq) '# HARMONICS BEFORE ALIAS OCCURS
      If Nharm < Nharm_min Then  'NEED TO INCREASE SAMPLE FREQUENCY
         Nharm = Nharm_min
      End If
      'FORCE ALIAS TO OCCUR EXACTLY AT Nharm*Freq !!
      TSampTemp = 0.0000001 * Int(1 / 2 / Nharm / freq / 0.0000001 + 0.5)
      burst_time = submeas_time * TSampTemp / (0.0015 + TSampTemp)
      Ncycle = Int(burst_time * freq + 1)
      num = Int((Ncycle / freq / TSampTemp) + 0.5)
      If Ncycle > 1 Then
         K = Int(num / 20 / Nharm + 1)
      Else
         K = 0
      End If
      'NOW ALIAS OCCURS MUCH HIGHER THAN Nharm*Freq K WAS PICKED TO
      ' TRY AND MAKE  ALIAS ABOUT 10*Nharm*Freq
      TSamp = 0.0000001 * Int(Ncycle / freq / (num - K) / 0.0000001 + 0.5)
      If TSamp - TSampTemp < 0.0000001 Then TSamp = TSamp + 0.0000001
    End If
    aper = TSamp - 0.00003
    num = Int(Ncycle / freq / TSamp + 0.5)
    If aper > 1 Then aper = 1 'MAX APERTURE OF HP3458A
    If aper < 0.000001 Then  'MIN APERTURE OF HP3458A
      Beep
      msg = "A/D Aperture is too small!" & vbCrLf _
             & "Lower Target Aperture, Minimum harmonics," & vbCrLf _
             & "or source Frequency"
       MsgBox msg, vbExclamation, AppName
       Samp_parm = False
      ' "ERROR", "A/D APERTURE TOO SMALL, DECREASE Aper_targ, Nharm, or INPUT Freq"
    End If
 
End Function

Public Sub setCancelTrue()
    m_cancel = True
End Sub

